from django.apps import AppConfig


class AuthentikateConfig(AppConfig):
    """The Django application configuration for the authentikate app."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "authentikate"
